--------------------------------------------------------------------------------
-- Broker_ProfessionsMenu                                                     --
-- Author: Sanori/Pathur                                                      --
--------------------------------------------------------------------------------
local _, me = ...                                 --Includes all functions and variables
me.version = "Version: 2.0.0 (16/01/2010)"



--------------------------------------------------------------------------------
-- Events, Libs, Variables                                                    --
--------------------------------------------------------------------------------
--Register Events
me.frame = CreateFrame("FRAME")
me.frame:RegisterEvent("ADDON_LOADED")
me.frame:RegisterEvent("PLAYER_LOGIN")
me.frame:RegisterEvent("PLAYER_LOGOUT")
me.frame:RegisterEvent("PLAYER_REGEN_DISABLED")
me.frame:RegisterEvent("TRADE_SKILL_UPDATE")
me.frame:RegisterEvent("TRADE_SKILL_SHOW")
me.frame:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED")
me.frame:RegisterEvent("UNIT_SPELLCAST_FAILED")
me.frame:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")
me.frame:RegisterEvent("LOOT_CLOSED")
me.frame:RegisterEvent("BAG_UPDATE")

--Include Libs
me.LDB = LibStub:GetLibrary("LibDataBroker-1.1")	--Data Broker
me.dewdrop = LibStub('ArkDewdrop-3.0')			--Dropdownmenu
me.TipHooker = LibStub("LibTipHooker-1.1")		--Tooltip mod
me.libst = LibStub("ScrollingTable")

--Variables
me.quicklauncher = {}					--Quick Launcher
me.sharedcds = {}					--shared cds
me.quicktradeskills = {}				--Quicktradeskills
BPMAutoloot=nil						--Global Variable for Auto Loot Script

if me.P.SharedCDs then
	for id, name in pairs(me.P.SharedCDs) do
		if GetSpellInfo(id) then 		--check if spell exists
			me.sharedcds[GetSpellInfo(id)] = name
		end
	end
end

--Localize TrainerDB
me.TrainerDBLoc = {}
for id,content in pairs(me.TrainerDB) do
	me.TrainerDBLoc[GetSpellInfo(id)] = content
end
me.TrainerDB = nil


--------------------------------------------------------------------------------
-- Functions                                                                  --
--------------------------------------------------------------------------------
--Filter Tradewindows and Tradeskills
function me:GetProfs(sorted)
	local result={}
	--Ignorelist
	local ignore={[GetSpellInfo(28677)]=true}	--Elixir Master
	--Additional
	local additional={--UnitClass("player") create atomaically the correct title
		[53428]=UnitClass("player"),		--Rune Forging
		[1804]=UnitClass("player"),		--Pick Look
	}
	--Professions
	for _,index in pairs({GetProfessions()}) do
		if index then
			local name, texture, rank, maxRank, numSpells, spelloffset, skillLine = GetProfessionInfo(index)
			if numSpells == 1 or numSpells == 2 and not ignore[name] then
				if not IsPassiveSpell(spelloffset+1, BOOKTYPE_PROFESSION) then
					local subname,_ = GetSpellBookItemName(spelloffset+1, BOOKTYPE_PROFESSION)
					if not ignore[subname] then
						if sorted then
							if not result[name] then result[name]={} end
							result[name][subname] = GetSpellBookItemTexture(spelloffset+1, BOOKTYPE_PROFESSION)
						else
							result[subname] = GetSpellBookItemTexture(spelloffset+1, BOOKTYPE_PROFESSION)
						end
					end
				end
				if numSpells == 2 and not IsPassiveSpell(spelloffset+2, BOOKTYPE_PROFESSION) then
					local subname,_ = GetSpellBookItemName(spelloffset+2, BOOKTYPE_PROFESSION)
					if not ignore[subname] then
						if sorted then
							if not result[name] then result[name]={} end
							result[name][subname] = GetSpellBookItemTexture(spelloffset+2, BOOKTYPE_PROFESSION)
						else
							result[subname] = GetSpellBookItemTexture(spelloffset+2, BOOKTYPE_PROFESSION)
						end
					end
				end
			end
		end
	end
	--Additional
	for id,title in pairs(additional) do
		local name,_,icon,_ = GetSpellInfo(id)
		if GetSpellBookItemInfo(name) then
			if sorted then
				if not result[title] then result[title]={} end
				result[title][name] = icon
			else
				result[name] = icon
			end
		end
	end
	return result
end
--Modified Version of GetSpellInfo()
function me:GetSpellInfo(spell)
	local name,_,icon,_ = GetSpellInfo(spell)
	if not name then name = "---" end
	if not icon then icon = "Interface\\Icons\\Trade_BlackSmithing" end
	return name, icon
end
--Table Alphabetical Sort Function
function me:pairsByKeys (t, f)
	local a = {}
	for n in pairs(t) do table.insert(a, n) end
		table.sort(a, f)
		local i = 0                                    --iterator variable
		local iter = function ()                       --iterator function
		i = i + 1
		if a[i] == nil then
			return nil
		else
			return a[i], t[a[i]]
		end
	end
	return iter
end
--Count of Table Elements
function me:tcount(tab)
	local n = #tab
	if (n == 0) then
		for _ in pairs(tab) do
			n = n + 1
		end
	end
	return n
end
--Save/Load
function me:SetSave(value,cat,subcat,player)
	if not player then player = UnitName("player") end
	if not me.save[player] then me.save[player]={} end
	if not cat then
		me.save[player]=value
	elseif not subcat then
		me.save[player][cat]=value
	else
		if not me.save[player][cat] then me.save[player][cat]={} end
		me.save[player][cat][subcat] = value
	end
end
function me:GetSave(cat,subcat,player)
	if not player then player = UnitName("player") end
	if not me.save[player] then me.save[player]={} end
	if not cat then
		return me.save[player]
	elseif not subcat then
		return me.save[player][cat]
	else
		if not me.save[player][cat] then me.save[player][cat]={} end
		return me.save[player][cat][subcat]
	end
end
--GetSpellId
function me:GetSpellID(spell)
	if not spell then return end
	local link,_ = GetSpellLink(spell)
	if not link then return end
	return tonumber(strmatch(link,"|Hspell:(%d+)|h"))
end



--------------------------------------------------------------------------------
-- Load/Save Saved Variables                                                  --
--------------------------------------------------------------------------------
me.frame:SetScript("OnEvent", function(self, event, ...)
	local arg1, arg2 = ...
	if event == "ADDON_LOADED" and arg1 == "Broker_ProfessionsMenu" then
		--Default values
		if (Broker_ProfessionsMenu == nil) then Broker_ProfessionsMenu = {} end
		if (Broker_ProfessionsMenu[GetCVar("realmName")] == nil) then Broker_ProfessionsMenu[GetCVar("realmName")] = {} end
		--Copy saves to internal var and initializise
		me.save = Broker_ProfessionsMenu[GetCVar("realmName")]
		if not me:GetSave("config") then me:SetSave({exchangeleftright=false},"config") end
		if not me:GetSave("bothfactions") then me:SetSave(false,"bothfactions") end
		if not me:GetSave("tooltip") then me:SetSave({showcds=true,showskills=true,showbuttons=true},"tooltip") end
		if not me:GetSave("ShowAllTooltips") then me:SetSave(true,"ShowAllTooltips") end
		if not me:GetSave("quicklaunch") then me:SetSave({left=0, shiftleft=0, altleft=0, ctrlleft=0},"quicklaunch") end
		if not me:GetSave("cds") then me:SetSave({},"cds") end
		if not me:GetSave("tradelinks") then me:SetSave({},"tradelinks") end
		if not me:GetSave("craftableitems") then me:SetSave({},"craftableitems") end
		if not me:GetSave("quicklauncher") then me:SetSave({},"quicklauncher") end
		if not me:GetSave("favorites") then me:SetSave({},"favorites") end
		if not me:GetSave("filter") then me:SetSave({},"filter") end
		if not me:GetSave("learnable") then me:SetSave({},"learnable") end
		if not me:GetSave("trainerdisabled") then me:SetSave(false,"trainerdisabled") end
	elseif event == "PLAYER_LOGIN" then
		--Init LibDataBroker
		me:InitLDB(me:GetSpellInfo(me:GetSave("quicklaunch","left")))
		--Create Launchers 
		for k,_ in pairs(me:GetSave("quicklauncher")) do
			me.quicklauncher[k]=me:newlauncher(me:GetSpellInfo(k))
		end
		--Modifying Atlasloot Recept Tooltips
		if AtlasLootTooltip then
			AtlasLootTooltip:HookScript("OnTooltipSetSpell", function(self, ...)
				me:modifytooltip(self,select(3,self:GetSpell()))
			end)
		end
		local faction,_=UnitFactionGroup("player")
		me:SetSave(faction,"faction")
	elseif event == "PLAYER_LOGOUT" then
		Broker_ProfessionsMenu[GetCVar("realmName")] = me.save
	elseif event == "PLAYER_REGEN_DISABLED" then
		me.secureframe:SetAttribute("*type1", nil)
		me.secureframe:SetAttribute("*type2", nil)
		for _,v in pairs(me.quicklauncher) do
			v.secureframe:SetAttribute("type1", nil)
		end
	elseif event == "TRADE_SKILL_SHOW" then
		--Trainer Data--------------------------------------------------
		if not me.Trainer then --Create Scroll List (call only once)
			me.TrainerFrame = CreateFrame("Frame","BPM_TradeSkillFrame",TradeSkillFrame)
			me.TrainerFrame:SetBackdrop({
				bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
				edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
				tile = true, tileSize = 16, edgeSize = 16,
				insets = { left = 3, right = 3, top = 5, bottom = 3 }
			})
			me.TrainerFrame:SetBackdropColor(0.1,0.1,0.1,0.6);
			me.TrainerFrame:SetFrameStrata(TradeSkillFrame:GetFrameStrata())
			me.TrainerFrame:SetFrameLevel(TradeSkillFrame:GetFrameLevel())
			me.Trainer = me.libst:CreateST({
				{
					--["name"] = "",
					["width"]=30,
					["align"]="center",
				},
				{
					["name"] = "|cffffcf00"..me.L.name.."|r",
					["width"]=200,
					["align"]="left",
					["defaultsort"] = "dsc",
				},
				{
					["name"] = "|cffffcf00"..me.L.skill.."|r",
					["width"]=32,
					["align"]="right",
					["defaultsort"] = "dsc",
					["sort"] = "desc",
					["sortnext"] = 2,
				},
			},12,31,{["r"] = 0,["g"] = 0.2,["b"] = 1.0,["a"] = 0.3},BPM_TradeSkillFrame)
			me.Trainer.frame:SetPoint("TOPLEFT",TradeSkillFrame,"TOPRIGHT",5,-37)
			me.Trainer.frame:SetPoint("BOTTOMLEFT",TradeSkillFrame,"BOTTOMRIGHT",5,5)
			me.TrainerFrame:SetPoint("TOPLEFT",TradeSkillFrame,"TOPRIGHT")
			me.TrainerFrame:SetPoint("BOTTOMRIGHT",me.Trainer.frame,"BOTTOMRIGHT",5,-5)
			me.TrainerFrame:Hide()
			me.Trainer:RegisterEvents({
				["OnEnter"] = function(rowFrame, cellFrame, data, cols, row, realrow, column, scrollingTable, ...)
					if not data or not row or not data[realrow] or not data[realrow]["cols"][2]["id"] then return end
					GameTooltip:SetOwner(rowFrame, "ANCHOR_NONE")
					GameTooltip:SetPoint("BOTTOMRIGHT",rowFrame,"TOPLEFT")
					GameTooltip:SetSpellByID(data[realrow]["cols"][2]["id"])
					GameTooltip:AddLine(" ")
					GameTooltip:AddLine(data[realrow]["cols"][2]["cost"])
					GameTooltip:Show()
				end,
				["OnLeave"] = function()
					GameTooltip:Hide()
				end,
			})
			me.Trainer.showbtn = CreateFrame("Button","BPM_ShowTrainerFrame",TradeSkillFrame,"UIPanelButtonTemplate")
			me.Trainer.showbtn:SetWidth(60)
			me.Trainer.showbtn:SetHeight(20)
			me.Trainer.showbtn:SetPoint("TOPRIGHT",TradeSkillFrame,"TOPRIGHT",-22,0)
			me.Trainer.showbtn:SetFrameStrata(TradeSkillDetailScrollChildFrame:GetFrameStrata())
			me.Trainer.showbtn:SetFrameLevel(TradeSkillDetailScrollChildFrame:GetFrameLevel()+1)
			me.Trainer.showbtn:SetText(me.L.trainer)
			me.Trainer.showbtn:SetScript("OnClick",function()
				me.Trainer.showframe = not me.Trainer.showframe
				me:Trainer_Refresh()
			end)
			me.Trainer.frame:Show()
			me.Trainer.showbtn:Hide()
			me.Trainer.showframe = nil
		end
		--Scann for Recipe----------------------------------------------
		if not IsTradeSkillLinked() and GetNumTradeSkills() and GetTradeSkillListLink() then
			local profid = me:GetSpellID(GetTradeSkillLine())
			me:SetSave("","craftableitems",profid)
			--Remove deleted Profession
			local craftableitems = me:GetSave("craftableitems")
			me:SetSave({},"craftableitems")
			for k,v in pairs(me:GetProfs()) do
				k = me:GetSpellID(k)
				if craftableitems[k]~=nil then
					me:SetSave(craftableitems[k],"craftableitems",k)
				end
			end
			--Scan Recepts
			for i = 1, GetNumTradeSkills() do
				local _,skilltype,_,isExpanded = GetTradeSkillInfo(i)
				if skilltype~="header" then
					local enchant=GetTradeSkillRecipeLink(i)
					local item=GetTradeSkillItemLink(i)
					if type(enchant)=="string" then
						enchant=strmatch(enchant,"|Henchant:(%d+)")
						if type(item)=="string" then
							item=strmatch(item,"|Hitem:(%d+):")
							if not item then item="0" end
						else
							item="0"
						end
						me:SetSave(me:GetSave("craftableitems",profid).."|"..enchant..","..item.."|","craftableitems",profid) 
					end
				else
					if not isExpanded then ExpandTradeSkillSubClass(i) end
				end
			end
			if not me:GetSave("trainerdisabled") then
				me:Trainer_Refresh()
				me.Trainer.showbtn:Show()
			end
		else
			me.Trainer.showbtn:Hide()
		end
		--Favorite Button-----------------------------------------------
		if not me.FavoriteButton then --Call only once
			me.FavoriteButton = CreateFrame("Button","BPM_TradeSkillFavoriteButton",TradeSkillFrame,"MagicButtonTemplate")
			--Replace Close Button
			me.FavoriteButton:SetWidth(TradeSkillCancelButton:GetWidth())
			me.FavoriteButton:SetHeight(TradeSkillCancelButton:GetHeight())
			me.FavoriteButton:SetFrameStrata(TradeSkillCancelButton:GetFrameStrata())
			me.FavoriteButton:SetFrameLevel(TradeSkillCancelButton:GetFrameLevel())
			me.FavoriteButton:SetPoint(TradeSkillCancelButton:GetPoint())
			--
			me.FavoriteButton:SetScript("OnClick",function(self)
				if not self.profid then return end
				local data = me:GetSave("favorites",self.profid)
				if not data then
					me:SetSave({},"favorites",self.profid)
					data = {}
				end
				if data[self.recipeid] then
					data[self.recipeid] = nil
					if me:tcount(data) == 0 then
						data = nil
					end
					me:SetSave(data,"favorites",self.profid)
					print(format(me.L.deletedfromfavorite,self.link))
				else
					data[self.recipeid] = self.type
					me:SetSave(data,"favorites",self.profid)
					print(format(me.L.addedtofavorite,self.link))
				end
				TradeSkillFrame_Update()
			end)
			--Hook Update Function
			hooksecurefunc("TradeSkillFrame_Update",function()
				if IsTradeSkillLinked() or not GetTradeSkillRecipeLink(GetTradeSkillSelectionIndex()) then
					me.FavoriteButton:Hide()
					TradeSkillCancelButton:Show()
				else
					local profid = me:GetSpellID(GetTradeSkillLine())
					if not profid then
						if GetTradeSkillLine()==GetSpellInfo(2575) then
							profid = 2656	--Verhüttung
						else
							me.FavoriteButton:Hide()
							TradeSkillCancelButton:Show()
							return
						end
					end
					local link = GetTradeSkillRecipeLink(GetTradeSkillSelectionIndex())
					local altVerb,_ = select(5,GetTradeSkillInfo(GetTradeSkillSelectionIndex()))
					local recipeid = tonumber(strmatch(link,"enchant:(%d+)"))
					if me:GetSave("favorites",profid) and me:GetSave("favorites",profid)[recipeid] then
						me.FavoriteButton:SetText(me.L.deletefromfavorite)
					else
						me.FavoriteButton:SetText(me.L.addtofavorite)
					end
					me.FavoriteButton.profid = profid
					me.FavoriteButton.recipeid = recipeid
					if altVerb then
						me.FavoriteButton.type = altVerb
					else
						me.FavoriteButton.type = "Create"
					end
					me.FavoriteButton.link = link
					TradeSkillCancelButton:Hide()
					me.FavoriteButton:Show()
				end
			end)
			TradeSkillFrame_Update()
		end
	--Scan Cooldowns--------------------------------------------------------
	elseif event == "TRADE_SKILL_UPDATE" then
		if not IsTradeSkillLinked() and GetNumTradeSkills()~=nil then
			if GetTradeSkillListLink()~=nil then
				me:SetSave(GetTradeSkillListLink(),"tradelinks",me:GetSpellID(strmatch(GetTradeSkillListLink(),"|h%[([%a%s]+)%]|h")))
				--Remove deleted Profession
				local tradelinks = me:GetSave("tradelinks")
				me:SetSave({},"tradelinks")
				for k,v in pairs(me:GetProfs()) do
					k = me:GetSpellID(k)
					if tradelinks[k]~=nil then
						me:SetSave(tradelinks[k],"tradelinks",k)
					end
				end
				--
			end
			for i = 1, GetNumTradeSkills() do
				local name,skilltype = GetTradeSkillInfo(i)
				if skilltype~="header" then
					local cd=GetTradeSkillCooldown(i)
					if cd then
						if me.sharedcds[name] then name=me.sharedcds[name] end
						me:SetSave(time()+floor(cd+0.5),"cds",name)
					end
				end
			end
		end
		if me.Trainer then me:Trainer_Refresh() end
	--Autoloot and Dropdownmenu Refresh
	elseif event == "UNIT_SPELLCAST_SUCCEEDED" then
		if arg1=="player" and BPMAutoloot then
			SetCVar('AutoLootDefault', 1)
		end
	elseif event == "UNIT_SPELLCAST_INTERRUPTED" or event == "UNIT_SPELLCAST_FAILED" then
		if arg1=="player" and BPMAutoloot then
			BPMAutoloot=nil
			SetCVar('AutoLootDefault', 0)
		end
	elseif event == "LOOT_CLOSED" then
		if BPMAutoloot then
			BPMAutoloot=nil
			SetCVar('AutoLootDefault', 0)
		end
	elseif event == "BAG_UPDATE" then
		if me.dewdrop:IsOpen(me.dewdrop.parent) then me.dewdrop:Refresh(2) end
	end
end)



--------------------------------------------------------------------------------
-- Modifying Tooltips                                                         --
--------------------------------------------------------------------------------
function me:modifytooltip(tooltip,id,isitem)
	if not id or not tooltip then return end
	local chars,prof
	for k,_ in pairs(me.save) do
		if me:GetSave("faction",nil,k)==UnitFactionGroup("player") or me:GetSave("bothfactions") then
			if me:GetSave("craftableitems",nil,k) then
				for kk,vv in pairs(me:GetSave("craftableitems",nil,k)) do
					local found
					if isitem then --Get ID of Henchant
						found = strfind(vv,"|%d+,"..id.."|") -- |ECHANT,ITEM|
					else
						found = strfind(vv,"|"..id..",%d+|")
					end
					if found then
						prof,_=GetSpellInfo(kk)
						if me:GetSave("bothfactions") and me:GetSave("faction",nil,k)=="Horde" then
							if not chars then chars=k.." ("..FACTION_HORDE..")" else chars=chars..", "..k.." ("..FACTION_HORDE..")" end
						elseif me:GetSave("bothfactions") and me:GetSave("faction",nil,k)=="Alliance" then
							if not chars then chars=k.." ("..FACTION_ALLIANCE..")" else chars=chars..", "..k.." ("..FACTION_ALLIANCE..")" end
						else
							if not chars then chars=k else chars=chars..", "..k end
						end
					end
				end
			end
		end
	end
	if prof then
		tooltip:AddLine(' ')
		tooltip:AddLine(prof..": "..chars,1,0.60,0)
		tooltip:Show()
	end
end
--Modifying Item Tooltips
me.TipHooker:Hook(function(self,...)
	 local itemid = strmatch(select(2,self:GetItem()),"|Hitem:(%d+):")
	 me:modifytooltip(self,itemid,true)
end, "item")
--Modifying Enchant Recept Tooltips
hooksecurefunc(ItemRefTooltip, "SetHyperlink", function(self, ...)
	local arg1,_ = ...
	if select(1,strfind(arg1,"enchant:")) then
		local id=strmatch(arg1,"enchant:(%d+)")
		me:modifytooltip(self,id)
	end
end)



--------------------------------------------------------------------------------
-- Init LibDataBroker (Main Window)                                           --
--------------------------------------------------------------------------------
function me:InitLDB(txt,icon)
	if txt=="---" then txt=me.L.professions end
	--Create Secure Frame Overlay
	me.secureframe = CreateFrame("Button", nil, UIParent, "SecureActionButtonTemplate");
	me.secureframe.OnMouseUp = function(self,button)
		if InCombatLockdown() then return end
		if me:GetSave("config","exchangeleftright") then
			self:SetAttribute("*type1", nil)
			self:SetAttribute("*type2", "spell")
		else
			self:SetAttribute("*type2", nil)
			self:SetAttribute("*type1", "spell")
		end
		if IsControlKeyDown() then
			self:SetAttribute("spell", GetSpellInfo(me:GetSave("quicklaunch","ctrlleft")))
		elseif IsAltKeyDown() then
			self:SetAttribute("spell", GetSpellInfo(me:GetSave("quicklaunch","altleft")))
		elseif IsShiftKeyDown() then
			self:SetAttribute("spell", GetSpellInfo(me:GetSave("quicklaunch","shiftleft")))
		else
			self:SetAttribute("spell", GetSpellInfo(me:GetSave("quicklaunch","left")))
		end
		if button == "RightButton" and not me:GetSave("config","exchangeleftright") then
			GameTooltip:Hide()
			me.dewdrop.parent=self
			if IsShiftKeyDown() then
				me.dewdrop:Open(self, 'children', function() me:ShowFavorites() end)
			else
				me.dewdrop:Open(self, 'children', function(level, value) me:UpdateMenu(level, value) end)
			end
		elseif button == "LeftButton" and me:GetSave("config","exchangeleftright") then
			GameTooltip:Hide()
			me.dewdrop.parent=self
			if IsShiftKeyDown() then
				me.dewdrop:Open(self, 'children', function() me:ShowFavorites() end)
			else
				me.dewdrop:Open(self, 'children', function(level, value) me:UpdateMenu(level, value) end)
			end
		end
	end
	--Create LDB
	me.dataobj = me.LDB:NewDataObject("Broker_ProfessionsMenu", {
		type = "data source",
		text = txt,
		icon = icon,
		OnEnter = function(self)
			me:Broker_OnEnter(self,me.secureframe,function(self) me:tooltip(self) end)
		end,
	})
end



--------------------------------------------------------------------------------
-- Creates Launchers                                                          --
--------------------------------------------------------------------------------
function me:newlauncher(spell,icon)
	local launcher = {}
	--secure frame
	launcher.secureframe = CreateFrame("Button", nil, UIParent, "SecureActionButtonTemplate")
	launcher.secureframe.OnMouseUp = function(self,button)
		if InCombatLockdown() then return end
		self:SetAttribute("type1", "spell")
		self:SetAttribute("spell", spell)
	end
	--Create Dataobject
	launcher.dataobj = me.LDB:NewDataObject("Launcher: "..spell, {
		type = "data source",
		text = spell,
		icon = icon,
		OnEnter = function(self)
			me:Broker_OnEnter(self,launcher.secureframe,function(self)
				local link=GetSpellLink(spell)
				if link then
					self:SetHyperlink(link)
				else
					self:AddLine(spell)
				end
			end)
		end
	})
	return launcher
end



--------------------------------------------------------------------------------
-- Refresh broker icon/text                                                   --
--------------------------------------------------------------------------------
function me:Professions_UpdateInfo(txt,icon)
	if txt=="---" then txt=me.L.professions end
	me.dataobj.text = txt
	me.dataobj.icon = icon
end



--------------------------------------------------------------------------------
-- Error Messages / Other Messages                                            --
--------------------------------------------------------------------------------
function me:Error(txt)
	UIErrorsFrame:AddMessage(txt,1.0,0,0,100,3);
end



--------------------------------------------------------------------------------
-- Secure Frame Stuff                                                         --
--------------------------------------------------------------------------------
function me:Broker_OnEnter(brokerframe,secureframe,tooltipfunc)
	if InCombatLockdown() then return end
	--Hover Events
	secureframe:SetScript("OnEnter",function(self)
		if brokerframe:GetScript("OnEnter") then
			brokerframe:GetScript("OnEnter")(brokerframe)
		end
		if not InCombatLockdown() and tooltipfunc and not self:GetScript("OnUpdate") and brokerframe:IsVisible() and me:GetSave("ShowAllTooltips") then
			GameTooltip:SetOwner(self, "ANCHOR_NONE")
			GameTooltip:SetPoint(me:GetTipAnchor(self))
			GameTooltip:ClearLines()
			tooltipfunc(GameTooltip)
			GameTooltip:Show()
		end
	end)
	secureframe:SetScript("OnLeave",function(self)
		GameTooltip:Hide()
		if brokerframe:GetScript("OnLeave") then
			brokerframe:GetScript("OnLeave")(brokerframe)
		end
	end)
	brokerframe:SetScript("OnHide",function()
		if not InCombatLockdown() then
			secureframe:Hide()
		end
	end)
	--Click Events
	secureframe:RegisterForClicks("AnyUp")
	secureframe:SetScript("OnMouseDown",function(self,button,down)
		if self.OnMouseUp and brokerframe:IsVisible() then
			self.OnMouseUp(self,button)
		end
	end)
	--Drag Frame
	secureframe:RegisterForDrag("LeftButton")
	secureframe:SetMovable(true)
	secureframe:SetScript("OnDragStart",function(self)
		if InCombatLockdown() then return end
		if brokerframe:GetScript("OnDragStart") then
			--if not brokerframe:IsMovable() then return end
			GameTooltip:Hide()
			brokerframe:GetScript("OnDragStart")(brokerframe)
			self:SetScript("OnUpdate", function(self)
				self:SetAllPoints(brokerframe)
				if not IsMouseButtonDown("LeftButton") then
					brokerframe:GetScript("OnDragStop")(brokerframe)
					self:SetScript("OnUpdate",nil)
				end
			end)
			self:Show()
		end
	end)
	--Show Frame
	--secureframe:SetNormalTexture("Interface\\Icons\\INV_Misc_QuestionMark")	--For Debug
	secureframe:SetFrameStrata(brokerframe:GetFrameStrata())
	secureframe:SetFrameLevel(brokerframe:GetFrameLevel()+1)
	secureframe:SetAllPoints(brokerframe)
	secureframe:Show()
end


--
function me:Trainer_Refresh()
	--if activatet
	if not IsTradeSkillLinked() and GetNumTradeSkills() and GetTradeSkillListLink() and me.Trainer.showframe and not me:GetSave("trainerdisabled") then
		local prof,curskill,maxskill = GetTradeSkillLine()
		local profid = me:GetSpellID(prof)
		local table={}
		if me.TrainerDBLoc[prof] then
			for reqskill,v in me:pairsByKeys(me.TrainerDBLoc[prof]) do
				for id,money in pairs(v) do
					if not strfind(me:GetSave("craftableitems",profid),"|"..tostring(id)..",") then --Test, if skill know
						local name,rank,icon,_ = GetSpellInfo(id)
						if ((strlen(rank)>0 and reqskill+25>=maxskill) or strlen(rank)==0) then
							local skill=""
							if reqskill>curskill then
								skill = "|cffff0000"..tostring(reqskill).."|r"
							else
								skill = "|cff00ff00"..tostring(reqskill).."|r"
							end
							local gold = floor( money / COPPER_PER_GOLD )
							local silver = floor( ( money - ( gold * COPPER_PER_GOLD ) ) / COPPER_PER_SILVER )
							local copper = mod( money, COPPER_PER_SILVER )
							local data={
								["cols"] = {
									{
										["value"] = "|T"..icon..":32:32:0:0|t",
									},--Spalte 1
									{
										["value"] = "|cffffcf00"..name.."|r "..rank,
										["id"] = id,
										["cost"] = format("%s %s %s",format( GOLD_AMOUNT_TEXTURE, gold, 0, 0 ),format( SILVER_AMOUNT_TEXTURE, silver, 0, 0 ),format( COPPER_AMOUNT_TEXTURE, copper, 0, 0 )),
									},--Spalte 2
									{
										["value"] = skill,
									},--Spalte 3
								},--Spalten
							}--Zeile
							tinsert(table,data)
						end
					end
				end
			end
			if me:tcount(table)==0 then
				local data={["cols"] = {{["value"] = "",},{["value"] = me.L.knowallrecipes,},{["value"] = "",},},}
				tinsert(table,data)
			end
			me.Trainer:SetData(table)
			me.TrainerFrame:Show()
			me.Trainer:SortData();
		else
			me.TrainerFrame:Hide()
		end
	else
		me.TrainerFrame:Hide()
	end
end